# copyright 2016 LOGILAB S.A. (Paris, FRANCE), all rights reserved.
# contact http://www.logilab.fr -- mailto:contact@logilab.fr
#
# This program is free software: you can redistribute it and/or modify it under
# the terms of the GNU Lesser General Public License as published by the Free
# Software Foundation, either version 2.1 of the License, or (at your option)
# any later version.
#
# This program is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
# FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more
# details.
#
# You should have received a copy of the GNU Lesser General Public License along
# with this program. If not, see <http://www.gnu.org/licenses/>.
"""Search facets for cubicweb-seda."""

from cubicweb.predicates import is_instance
from cubicweb.web.facet import AttributeFacet


class SimplifiedProfileFacet(AttributeFacet):
    __regid__ = 'seda.profile.simplified'
    __select__ = AttributeFacet.__select__ & is_instance('SEDAArchiveTransfer')
    rtype = 'simplified_profile'

    def vocabulary(self):
        __ = self._cw.__
        return [(__('yes'), True), (__('no'), False)]
