/* add span/a to override default rules */

ul.jqtree-tree span.jqtree-title {
  margin-left: 0;
}

ul.jqtree-tree i.glyphicon {
  font-size: 80%;
  color: #777;
}

ul.jqtree-tree a.jqtree-toggler.jqtree-toggler-left {
  margin-right: 0.1em;
}